@@ -3,9 +3,7 @@
 #
 #  Method for 'density' for point patterns
 #
-#  + bandwidth selection rules bw.diggle, bw.scott
-#
-#  $Revision: 1.65 $    $Date: 2014/01/19 11:09:10 $
+#  $Revision: 1.69 $    $Date: 2014/05/20 09:30:40 $
 #
 
 ksmooth.ppp <- function(x, sigma, ..., edge=TRUE) {
@@ -143,7 +141,7 @@ densitypointsEngine <- function(x, sigma, ...,
 #  }
   if(leaveoneout) {
     # ensure each point has its closest neighbours within the cutoff
-    nndmax <- max(nndist(x))
+    nndmax <- maxnndist(x)
     cutoff <- max(2 * nndmax, cutoff)
   }
   # validate weights
@@ -213,7 +211,6 @@ densitypointsEngine <- function(x, sigma, ...,
                  sig     = as.double(sd),
                  result  = as.double(double(npts)),
                  DUP     = DUP)
-#                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else if(k == 1) {
         wtsort <- if(sorted) weights else weights[oo]
@@ -226,7 +223,6 @@ densitypointsEngine <- function(x, sigma, ...,
                  weight  = as.double(wtsort),
                  result  = as.double(double(npts)),
                  DUP     = DUP)
-#                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
        } else {
         # matrix of weights
@@ -241,7 +237,6 @@ densitypointsEngine <- function(x, sigma, ...,
                    weight  = as.double(wtsort[,j]),
                    result  = as.double(double(npts)),
                    DUP     = DUP)
-#                   PACKAGE = "spatstat")
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result
         }
       }
@@ -258,7 +253,6 @@ densitypointsEngine <- function(x, sigma, ...,
                  sinv    = as.double(flatSinv),
                  result  = as.double(double(npts)),
                  DUP     = DUP)
-#                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else if(k == 1) {
         # vector of weights
@@ -273,7 +267,6 @@ densitypointsEngine <- function(x, sigma, ...,
                  weight  = as.double(wtsort),
                  result   = as.double(double(npts)),
                  DUP     = DUP)
-#                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else {
         # matrix of weights
@@ -289,7 +282,6 @@ densitypointsEngine <- function(x, sigma, ...,
                    weight  = as.double(wtsort[,j]),
                    result  = as.double(double(npts)),
                    DUP     = DUP)
-#                   PACKAGE = "spatstat")
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result 
         }
       }
@@ -432,71 +424,208 @@ resolve.2D.kernel <- function(..., sigma=NULL, varcov=NULL, x, mindist=NULL,
   return(result)
 }
 
-bw.diggle <- function(X, ...) {
-  stopifnot(is.ppp(X))
-  # secret option for debugging
-  mf <- function(..., method=c("C", "interpreted")) match.arg(method)
-  method <- mf(...)
-  #
-  lambda <- npoints(X)/area.owin(X)
-  K <- Kest(X, correction="good")
-  yname <- fvnames(K, ".y")
-  K <- K[, c("r", yname)]
-  rvals <- K$r
-  # evaluation of M(r) requires K(2r)
-  rmax2 <- max(rvals)/2
-  if(!is.null(alim <- attr(K, "alim"))) rmax2 <- min(alim[2], rmax2)
-  ok <- (rvals <= rmax2)
-  switch(method,
-         interpreted = {
-           rvals <- rvals[ok]
-           nr <- length(rvals)
-           J <- numeric(nr)
-           phi <- function(x,h) { 
-             if(h <= 0) return(numeric(length(x)))
-             y <- pmax.int(0, pmin.int(1, x/(2 * h)))
-             4 * pi * h^2 * (acos(y) - y * sqrt(1 - y^2))
-           }
-           for(i in 1:nr) 
-             J[i] <- stieltjes(phi, K, h=rvals[i])[[yname]]/(2 * pi)
-         },
-         C = {
-           nr <- length(rvals)
-           nrmax <- sum(ok)
-           dK <- diff(K[[yname]])
-           ndK <- length(dK)
-           DUP <- spatstat.options("dupC")
-           z <- .C("digberJ",
-                   r=as.double(rvals),
-                   dK=as.double(dK),
-                   nr=as.integer(nr),
-                   nrmax=as.integer(nrmax),
-                   ndK=as.integer(ndK),
-                   J=as.double(numeric(nrmax)),
-                   DUP=DUP)
-#                  PACKAGE="spatstat"              
-           J <- z$J
-           rvals <- rvals[ok]
-         })
-  pir2 <- pi * rvals^2
-  M <- (1/lambda - 2 * K[[yname]][ok])/pir2 + J/pir2^2
-  # This calculation was for the uniform kernel on B(0,h)
-  # Convert to standard deviation of (one-dimensional marginal) kernel
-  sigma <- rvals/2
-  result <- bw.optim(M, sigma,
-                     creator="bw.diggle",
-                     criterion="Berman-Diggle Cross-Validation",
-                     J=J,
-                     lambda=lambda)
+
+densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
+                               weights=NULL, edge=TRUE, varcov=NULL,
+                               diggle=FALSE,
+                               sorted=FALSE) {
+  if(is.null(varcov)) {
+    const <- 1/(2 * pi * sigma^2)
+  } else {
+    detSigma <- det(varcov)
+    Sinv <- solve(varcov)
+    const <- 1/(2 * pi * sqrt(detSigma))
+  }
+  if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
+    weights <- NULL
+  ## Leave-one-out computation
+  ## cutoff: contributions from pairs of distinct points
+  ## closer than 8 standard deviations
+  sd <- if(is.null(varcov)) sigma else sqrt(sum(diag(varcov)))
+  cutoff <- 8 * sd
+  # validate weights
+  if(is.null(weights)) {
+    k <- 1
+  } else if(is.matrix(weights) || is.data.frame(weights)) {
+    k <- ncol(weights)
+    stopifnot(nrow(weights) == npoints(Xdata))
+    weights <- as.data.frame(weights)
+    weightnames <- colnames(weights)
+  } else {
+    k <- 1
+    stopifnot(length(weights) == npoints(Xdata) || length(weights) == 1)
+  }
+  # evaluate edge correction weights at points 
+  if(edge) {
+    win <- Xdata$window
+    if(diggle) {
+      ## edge correction weights are attached to data points
+      xedge <- Xdata
+    } else {
+      ## edge correction weights are applied at query points
+      xedge <- Xquery
+      if(!all(inside.owin(Xquery, , win)))
+        stop(paste("Edge correction is not possible:",
+                   "some query points lie outside the data window"),
+             call.=FALSE)
+    }
+    if(is.null(varcov) && win$type == "rectangle") {
+        ## evaluate Gaussian probabilities directly
+      xr <- win$xrange
+      yr <- win$yrange
+      xx <- xedge$x
+      yy <- xedge$y
+      xprob <-
+        pnorm(xr[2], mean=xx, sd=sigma) - pnorm(xr[1], mean=xx, sd=sigma)
+      yprob <-
+        pnorm(yr[2], mean=yy, sd=sigma) - pnorm(yr[1], mean=yy, sd=sigma)
+      edgeweight <- xprob * yprob
+    } else {
+      edg <- second.moment.calc(Xdata, sigma=sigma,
+                                what="edge", varcov=varcov)
+      edgeweight <- safelookup(edg, xedge, warn=FALSE)
+    }
+    if(diggle) {
+      ## Diggle edge correction
+      ## edgeweight is attached to each data point
+      if(is.null(weights)) {
+        k <- 1
+        weights <- 1/edgeweight
+      } else {
+        weights <- weights/edgeweight
+      }
+    }
+  }
+  
+  ndata <- npoints(Xdata)
+  nquery <- npoints(Xquery)
+  result <- if(k == 1) numeric(nquery) else matrix(, nquery, k)
+  ## coordinates
+  xq <- Xquery$x
+  yq <- Xquery$y
+  xd <- Xdata$x
+  yd <- Xdata$y
+  if(!sorted) {
+    ## sort into increasing order of x coordinate (required by C code)
+    ooq <- fave.order(Xquery$x)
+    xq <- xq[ooq]
+    yq <- yq[ooq]
+    ood <- fave.order(Xdata$x)
+    xd <- xd[ood]
+    yd <- yd[ood]
+  }
+  DUP <- spatstat.options("dupC")
+  if(is.null(varcov)) {
+    ## isotropic kernel
+    if(is.null(weights)) {
+      zz <- .C("crdenspt",
+               nquery  = as.integer(nquery),
+               xq      = as.double(xq),
+               yq      = as.double(yq),
+               ndata   = as.integer(ndata),
+               xd      = as.double(xd),
+               yd      = as.double(yd),
+               rmaxi   = as.double(cutoff),
+               sig     = as.double(sd),
+               result  = as.double(double(nquery)),
+               DUP     = DUP)
+      if(sorted) result <- zz$result else result[ooq] <- zz$result 
+    } else if(k == 1) {
+      wtsort <- if(sorted) weights else weights[ood]
+      zz <- .C("wtcrdenspt",
+               nquery  = as.integer(nquery),
+               xq      = as.double(xq),
+               yq      = as.double(yq),
+               ndata   = as.integer(ndata),
+               xd      = as.double(xd),
+               yd      = as.double(yd),
+               wd      = as.double(wtsort),
+               rmaxi   = as.double(cutoff),
+               sig     = as.double(sd),
+               result  = as.double(double(nquery)),
+               DUP     = DUP)
+      if(sorted) result <- zz$result else result[ooq] <- zz$result 
+    } else {
+      ## matrix of weights
+      wtsort <- if(sorted) weights else weights[ood, ]
+      for(j in 1:k) {
+        zz <- .C("wtcrdenspt",
+                 nquery  = as.integer(nquery),
+                 xq      = as.double(xq),
+                 yq      = as.double(yq),
+                 ndata   = as.integer(ndata),
+                 xd      = as.double(xd),
+                 yd      = as.double(yd),
+                 wd      = as.double(wtsort[,j]),
+                 rmaxi   = as.double(cutoff),
+                 sig     = as.double(sd),
+                 result  = as.double(double(nquery)),
+                 DUP     = DUP)
+        if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result
+      }
+    }
+  } else {
+    ## anisotropic kernel
+    flatSinv <- as.vector(t(Sinv))
+    if(is.null(weights)) {
+      zz <- .C("acrdenspt",
+               nquery  = as.integer(nquery),
+               xq      = as.double(xq),
+               yq      = as.double(yq),
+               ndata   = as.integer(ndata),
+               xd      = as.double(xd),
+               yd      = as.double(yd),
+               rmaxi   = as.double(cutoff),
+               detsigma = as.double(detSigma),
+               sinv    = as.double(flatSinv),
+               result  = as.double(double(nquery)),
+               DUP     = DUP)
+      if(sorted) result <- zz$result else result[ooq] <- zz$result 
+    } else if(k == 1) {
+      ## vector of weights
+      wtsort <- if(sorted) weights else weights[ood]
+      zz <- .C("awtcrdenspt",
+               nquery  = as.integer(nquery),
+               xq      = as.double(xq),
+               yq      = as.double(yq),
+               ndata   = as.integer(ndata),
+               xd      = as.double(xd),
+               yd      = as.double(yd),
+               wd      = as.double(wtsort),
+               rmaxi   = as.double(cutoff),
+               detsigma = as.double(detSigma),
+               sinv    = as.double(flatSinv),
+               result   = as.double(double(nquery)),
+               DUP     = DUP)
+      if(sorted) result <- zz$result else result[ooq] <- zz$result 
+    } else {
+      ## matrix of weights
+      wtsort <- if(sorted) weights else weights[ood, ]
+      for(j in 1:k) {
+        zz <- .C("awtcrdenspt",
+                 nquery  = as.integer(nquery),
+                 xq      = as.double(xq),
+                 yq      = as.double(yq),
+                 ndata   = as.integer(ndata),
+                 xd      = as.double(xd),
+                 yd      = as.double(yd),
+                 wd      = as.double(wtsort[,j]),
+                 rmaxi   = as.double(cutoff),
+                 detsigma = as.double(detSigma),
+                 sinv    = as.double(flatSinv),
+                 result  = as.double(double(nquery)),
+                 DUP     = DUP)
+        if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result 
+      }
+    }
+  }
+  # ........  Edge correction ........................................
+  if(edge && !diggle) 
+    result <- result/edgeweight
+
+  # tack on bandwidth
+  attr(result, "sigma") <- sigma
+  attr(result, "varcov") <- varcov
+  # 
   return(result)
 }
-
-bw.scott <- function(X) {
-  stopifnot(is.ppp(X))
-  n <- npoints(X)
-  sdx <- sqrt(var(X$x))
-  sdy <- sqrt(var(X$y))
-  return(c(sdx, sdy) * n^(-1/6))
-}
-
-
