@@ -8,7 +8,7 @@
 #        harmonise.im()       Harmonise images
 #        commonGrid()
 #
-#     $Revision: 1.30 $     $Date: 2013/04/25 06:37:43 $
+#     $Revision: 1.32 $     $Date: 2014/07/29 09:04:19 $
 #
 
 eval.im <- function(expr, envir, harmonize=TRUE) {
@@ -20,8 +20,11 @@ eval.im <- function(expr, envir, harmonize=TRUE) {
   if(length(varnames) == 0)
     stop("No variables in this expression")
   # get the values of the variables
-  if(missing(envir))
+  if(missing(envir)) {
     envir <- sys.parent()
+  } else if(is.list(envir)) {
+    envir <- list2env(envir, parent=parent.frame())
+  }
   vars <- lapply(as.list(varnames), function(x, e) get(x, envir=e), e=envir)
   names(vars) <- varnames
   funs <- lapply(as.list(funnames), function(x, e) get(x, envir=e), e=envir)
@@ -59,7 +62,9 @@ eval.im <- function(expr, envir, harmonize=TRUE) {
   v <- eval(e, append(vars, funs))
   #
   # reshape, etc
-  result <- im(v, template$xcol, template$yrow, 
+  result <- im(v,
+               xcol=template$xcol, yrow=template$yrow,
+               xrange=template$xrange, yrange=template$yrange, 
                unitname=unitname(template))
   return(result)
 }
@@ -101,7 +106,7 @@ harmonize.im <- harmonise.im <- function(...) {
   imgs <- argz[isim]
   # if any windows are present, extract bounding box
   iswin <- unlist(lapply(argz, is.owin))
-  bb0 <- if(!any(iswin)) NULL else do.call("bounding.box", unname(argz[iswin]))
+  bb0 <- if(!any(iswin)) NULL else do.call("boundingbox", unname(argz[iswin]))
   if(length(imgs) == 1 && is.null(bb0)) {
     # only one 'true' image: use it as template.
     result[isim] <- imgs
@@ -117,8 +122,8 @@ harmonize.im <- harmonise.im <- function(...) {
     which.finest <- which.min(xsteps)
     finest <- imgs[[which.finest]]
     # get the bounding box
-    bb <- do.call("bounding.box", lapply(unname(imgs), as.rectangle))
-    if(!is.null(bb0)) bb <- bounding.box(bb, bb0)
+    bb <- do.call("boundingbox", lapply(unname(imgs), as.rectangle))
+    if(!is.null(bb0)) bb <- boundingbox(bb, bb0)
     # determine new raster coordinates
     xcol <- prolongseq(finest$xcol, bb$xrange)
     yrow <- prolongseq(finest$yrow, bb$yrange)
@@ -178,7 +183,7 @@ commonGrid <- local({
       finest <- rasterlist[[which.finest]]
     }
     # determine the bounding box
-    bb <- do.call("bounding.box", lapply(unname(argz[haswin]), as.rectangle))
+    bb <- do.call("boundingbox", lapply(unname(argz[haswin]), as.rectangle))
     # determine new raster coordinates
     xcol <- prolongseq(finest$xcol, bb$xrange)
     yrow <- prolongseq(finest$yrow, bb$yrange)
@@ -217,6 +222,7 @@ im.apply <- function(X, FUN, ...) {
   d <- dim(template)
   resultmat <- matrix(resultok[1], d[1], d[2])
   resultmat[ok] <- resultok
+  resultmat[!ok] <- NA
   result <- as.im(resultmat, W=X[[1]])
   if(is.factor(resultok)) levels(result) <- levels(resultok)
   return(result)
