@@ -1,7 +1,7 @@
 #
 #           pixellate.R
 #
-#           $Revision: 1.9 $    $Date: 2013/07/26 01:55:24 $
+#           $Revision: 1.12 $    $Date: 2014/06/27 06:32:02 $
 #
 #     pixellate            convert an object to a pixel image
 #
@@ -23,10 +23,9 @@ pixellate.ppp <- function(x, W=NULL, ..., weights=NULL, padzero=FALSE) {
     W <- as.mask(W)
   else {
     # determine W using as.mask
-    dotargs <- list(...)
-    namesargs <- names(dotargs)
-    matched <- namesargs %in% names(formals(as.mask))
-    W <- do.call("as.mask", append(list(x$window), dotargs[matched]))
+    W <- do.call.matched("as.mask",
+                         resolve.defaults(list(...),
+                                          list(w=x$window)))
   }
   insideW <- W$m
   dimW   <- W$dim
@@ -43,7 +42,7 @@ pixellate.ppp <- function(x, W=NULL, ..., weights=NULL, padzero=FALSE) {
     weights <- if(k == 1) as.vector(weights) else as.data.frame(weights)
   } else {
     k <- 1
-    if(!is.null(weights))
+    if(length(weights) == 0) weights <- NULL else 
       stopifnot(length(weights) == npoints(x) || length(weights) == 1)
     if(length(weights) == 1)
       weights <- rep(weights, npoints(x))
@@ -161,12 +160,12 @@ pixellate.owin <- function(x, W=NULL, ...) {
              npoly=as.integer(nn),
              out=as.double(numeric(nx * ny)),
              status=as.integer(integer(1)),
-             DUP=DUP,
-             PACKAGE="spatstat")
+             DUP=DUP)
+#             PACKAGE="spatstat")
     if(zz$status != 0)
       stop("Internal error")
     # increment output image
-    Z$v <- Z$v + matrix(zz$out, ny, nx, byrow=TRUE)
+    Z$v <- Z$v + matrix(zz$out, ny, nx)
   }
   # revert to original scale
   pixelarea <- dx * dy
