@@ -1,7 +1,7 @@
 #
 # profilepl.R
 #
-#  $Revision: 1.18 $  $Date: 2013/06/18 01:38:42 $
+#  $Revision: 1.22 $  $Date: 2014/08/07 02:25:27 $
 #
 #  computes profile log pseudolikelihood
 #
@@ -127,8 +127,9 @@ profilepl <- function(s, f, ..., rbord=NULL, verbose=TRUE) {
   optint <- do.call("f", as.list(s[opti, is.farg, drop=FALSE]))
   optarg <- list(interaction=optint, ..., rbord=rbord)
   if(pass.cfa) {
-    optcfa <- list(covfunargs=as.list(s[opti, !is.farg, drop=FALSE]))
-    optarg <- append(optarg, optcfa)
+    optcfa <- as.list(s[opti, !is.farg, drop=FALSE])
+    attr(optcfa, "fitter") <- "profilepl"
+    optarg <- append(optarg, list(covfunargs=optcfa))
   }
   optfit <- do.call("ppm", optarg)
   if(verbose) message("done.")
@@ -150,27 +151,35 @@ profilepl <- function(s, f, ..., rbord=NULL, verbose=TRUE) {
 #
 
 print.profilepl <- function(x, ...) {
-  cat("Profile log pseudolikelihood values\n")
-  cat("for model:\t")
-  print(x$pseudocall)
-  cat(paste("fitted with rbord=", x$rbord, "\n"))
+  head1 <- "Profile log pseudolikelihood"
+  head2 <- "for model: "
+  psc <- paste(unlist(strsplitretain(format(x$pseudocall))),
+               collapse=" ")
+  if(nchar(psc) + nchar(head2) + 1 <= getOption('width')) {
+    splat(head1)
+    splat(head2, psc)
+  } else {
+    splat(head1, head2)
+    splat(psc)
+  }
   nparm <- ncol(x$param)
-  cat(paste("Interaction:", x$fname,
-            "\n", "with irregular",
-            ngettext(nparm, "parameter ", "parameters\n")))
-  for(na in names(x$param)) {
-    ra <- range(x$param[[na]])
-    cat(paste(sQuote(na), "in",
-              paste("[",ra[1],", ",ra[2],"]",sep=""),
-              "\n"))
+  if(waxlyrical('extras')) {
+    splat("fitted with rbord =", x$rbord)
+    splat("Interaction:", x$fname)
+    splat("Irregular",
+          ngettext(nparm, "parameter:", "parameters:\n"),
+          paste(names(x$param),
+                "in",
+                unlist(lapply(lapply(as.list(x$param), range), prange)),
+                collapse="\n"))
   }
-  cat(paste("Optimum",
-            ngettext(nparm, "value", "values"),
-            "of irregular",
-            ngettext(nparm, "parameter: ", "parameters:\n")))
   popt <- x$param[x$iopt,, drop=FALSE]
-  cat(commasep(paste(names(popt), "=", popt)))
-  cat("\n")
+  splat("Optimum",
+        ngettext(nparm, "value", "values"),
+        "of irregular",
+        ngettext(nparm, "parameter: ", "parameters:\n"),
+        commasep(paste(names(popt), "=", popt)))
+  invisible(NULL)
 }
 
 #
