@@ -3,26 +3,29 @@
 #
 #	Computes the GNZ contrast of delta-f for any function f
 #
-#	$Revision: 1.4 $	$Date: 2013/02/26 07:03:14 $
+#	$Revision: 1.6 $	$Date: 2014/06/17 04:43:13 $
 #
 ################################################################################
 #
 
 psst <- function(object, fun, r=NULL, breaks=NULL, ...,
+                 model=NULL,
                  trend=~1, interaction=Poisson(),
                  rbord=reach(interaction),
                  truecoef=NULL, hi.res=NULL,
                  funargs=list(correction="best"),
                  verbose=TRUE) {
-  if(inherits(object, "ppm")) 
+  if(inherits(object, "ppm")) {
     fit <- object
-  else if(inherits(object, "ppp"))
-    fit <- ppm(quadscheme(object, ...),
-               trend=trend, interaction=interaction, rbord=rbord)
-  else if(inherits(object, "quad")) 
-    fit <- ppm(object, 
-               trend=trend, interaction=interaction, rbord=rbord)
-  else 
+  } else if(is.ppp(object) || inherits(object, "quad")) {
+    if(is.ppp(object)) object <- quadscheme(object, ...)
+    if(!is.null(model)) {
+      fit <- update(model, Q=object, forcefit=TRUE)
+    } else {
+      fit <- ppm(object, trend=trend, interaction=interaction, rbord=rbord,
+                 forcefit=TRUE)
+    }
+  } else 
     stop("object should be a fitted point process model or a point pattern")
 
   rfixed <- !is.null(r) || !is.null(breaks)
@@ -42,7 +45,7 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
     rbord <- fit$rbord
     b <- bdist.points(U)
     USED <- (b > rbord)
-  } else USED <- rep(TRUE, U$n)
+  } else USED <- rep.int(TRUE, U$n)
   
   # basic statistics
   Win <- X$window
@@ -57,7 +60,7 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
     area.used <- sum(WQ[USED])
     lambda.used <- npoints.used/area.used
   } else {
-    XUSED <- rep(TRUE, npoints)
+    XUSED <- rep.int(TRUE, npoints)
     npoints.used <- npoints
     area.used <- area
     lambda.used <- lambda
@@ -71,7 +74,7 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
   
   # residuals
   resid <- residuals(fit, type="raw",drop=FALSE,
-                    coefs=truecoef, quad=hi.res)
+                    new.coef=truecoef, quad=hi.res)
   rescts <- with(resid, "continuous")
   # absolute weight for continuous integrals
   wc   <- -rescts
@@ -89,7 +92,7 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
   fXunits <- unitname(fX)
   # Extract 'best' estimate only
   fX <- with(fX, .y)
-  zero <- rep(0, length(fX))
+  zero <- numeric(length(fX))
   # sum over all quadrature points
   iused <- seq(U$n)[USED]
   nused <- length(iused)
