@@ -3,7 +3,7 @@
 #
 #    summary() method for class "ppm"
 #
-#    $Revision: 1.65 $   $Date: 2013/11/12 16:31:54 $
+#    $Revision: 1.70 $   $Date: 2014/08/14 08:27:54 $
 #
 #    summary.ppm()
 #    print.summary.ppm()
@@ -152,6 +152,7 @@ summary.ppm <- local({
         # do covariates depend on additional arguments?
         if(y$has.funcs) {
           y$covfunargs <- x$covfunargs
+          y$cfafitter <- attr(x$covfunargs, "fitter")
           funs <- covars[isfun]
           fdescrip <- function(f) {
             if(inherits(f, "distfun")) return("distfun")
@@ -250,7 +251,6 @@ summary.ppm <- local({
              })
     } else {
       # not stationary 
-      y$trend$label <- "Fitted coefficients for trend formula"
       # extract trend terms without trying to understand them much
       if(is.null(Vnames)) 
         trendbits <- COEFS
@@ -259,6 +259,9 @@ summary.ppm <- local({
         whichbits <- apply(!agree, 1, all)
         trendbits <- COEFS[whichbits]
       }
+      y$trend$label <- ngettext(length(trendbits),
+                                "Fitted trend coefficient",
+                                "Fitted trend coefficients")
       y$trend$value <- blankcoefnames(trendbits)
     }
   
@@ -281,15 +284,16 @@ summary.ppm <- local({
         two <- qnorm(0.975)
         lo <- COEFS - two * se
         hi <- COEFS + two * se
-        pval <- 2 * pnorm(abs(COEFS)/se, lower.tail=FALSE)
-        psig <- cut(pval, c(0,0.001, 0.01, 0.05, 1, Inf),
-                    labels=c("***", "**", "*", "  ", "na"),
+        zval <- COEFS/se
+        pval <- 2 * pnorm(abs(zval), lower.tail=FALSE)
+        psig <- cut(pval, c(0,0.001, 0.01, 0.05, 1),
+                    labels=c("***", "**", "*", "  "),
                     include.lowest=TRUE)
-        notapplic <- names(COEFS) %in% c("(Intercept)", "log(lambda)")
-        psig[notapplic] <- "na"
         # table of coefficient estimates with SE and 95% CI
-        y$coefs.SE.CI <- data.frame(Estimate=COEFS, S.E.=se, Ztest=psig,
-                                    CI95.lo=lo, CI95.hi=hi)
+        y$coefs.SE.CI <- data.frame(Estimate=COEFS, S.E.=se,
+                                    CI95.lo=lo, CI95.hi=hi,
+                                    Ztest=psig,
+                                    Zval=zval)
       }
     }
   
@@ -403,8 +407,8 @@ print.summary.ppm <- function(x, ...) {
   cat(paste("\n\n ---- ", x$trend$name, ": ----\n\n", sep=""))
 
   if(!notrend) {
-    cat("Trend formula: ")
-    print(x$trend$formula)
+    splat("Trend formula: ",
+          pasteFormula(x$trend$formula))
     if(x$uses.covars) 
       cat(paste("Model depends on external",
                 ngettext(length(x$covars.used), "covariate", "covariates"),
