@@ -1,7 +1,7 @@
 #
 #    util.S    miscellaneous utilities
 #
-#    $Revision: 1.132 $    $Date: 2014/01/09 06:53:08 $
+#    $Revision: 1.159 $    $Date: 2014/08/08 07:10:09 $
 #
 #
 matrowsum <- function(x) {
@@ -56,12 +56,21 @@ whist <- function(x, breaks, weights=NULL) {
     else {
       # classify data into histogram cells (breaks need not span range of data)
       cell <- findInterval(x, breaks, rightmost.closed=TRUE)
-      cell <- factor(cell, levels=0:N)
-      # compute weighted histogram
-      if(is.null(weights))
-        h <- table(cell)
-      else 
-        h <- unlist(lapply(split(weights, cell), sum, na.rm=TRUE))
+      # values of 'cell' range from 0 to N.
+      nb <- N + 1L
+      if(is.null(weights)) {
+        ## histogram
+        h <- tabulate(cell+1L, nbins=nb)
+      } else {
+        ##  weighted histogram
+        if(!spatstat.options("Cwhist")) {
+          cell <- factor(cell, levels=0:N)
+          h <- unlist(lapply(split(weights, cell), sum, na.rm=TRUE))
+        } else {
+          h <- .Call("Cwhist",
+                     as.integer(cell), as.double(weights), as.integer(nb))
+        }
+      }
     }
     h <- as.numeric(h)
     y <- h[2:N]
@@ -142,23 +151,23 @@ rastersample <- function(X, Y) {
 pointgrid <- function(W, ngrid) {
   W <- as.owin(W)
   masque <- as.mask(W, dimyx=ngrid)
-  xx <- raster.x(masque)
-  yy <- raster.y(masque)
-  xx <- xx[masque$m]
-  yy <- yy[masque$m]
+  rxy <- rasterxy.mask(masque, drop=TRUE)
+  xx <- rxy$x
+  yy <- rxy$y
   return(ppp(xx, yy, W))
 }
 
 # text magic
 
-commasep <- function(x, join="and") {
+commasep <- function(x, join=" and ", flatten=TRUE) {
   px <- paste(x)
   nx <- length(px)
   if(nx <= 1) return(px)
   commas <- c(rep(", ", length(px)-2),
-              paste("", join, ""),
+              join,
               "")
-  return(paste(paste(px, commas, sep=""), collapse=""))
+  out <- paste0(px, commas, collapse=if(flatten) "" else NULL)
+  return(out)
 }
 
 paren <- function(x, type="(") {
@@ -191,6 +200,40 @@ unparen <- function(x) {
   return(x)
 }
 
+strsplitretain <- local({
+  strsplitretain <- function(x, split=",") {
+    ## split strings after occurrence of character b, but retain b
+    y <- strsplit(x, split)
+    lapply(y, addback, b=split)
+  }
+  addback <- function(x, b=",") {
+    n <- length(x)
+    if(n <= 1) x else c(paste0(x[-n], b), x[n])
+  }    
+  strsplitretain
+})
+
+truncline <- function(x, nc) {
+  if(length(x) > 1)
+    return(unlist(lapply(as.list(x), truncline, nc=nc)))
+  ## split string into words
+  y <- strsplit(x, " ", fixed=TRUE)[[1]]
+  ## find max number of whole words that take up nc characters
+  maxwords <- max(0, which(cumsum(nchar(y) + 1) <= nc+1))
+  if(maxwords == length(y))
+    return(x)
+  ## truncation will occur.
+  pad <- " [..]"
+  nc <- nc - nchar(pad)
+  maxwords <- max(0, which(cumsum(nchar(y) + 1) <= nc+1))
+  z <- paste(y[seq_len(maxwords)], collapse=" ")
+  d <- nc - nchar(z)
+  if(d < 0)
+    z <- substr(z, 1, nc)
+  z <- paste0(z, pad)
+  return(z)
+}
+
 fakecallstring <- function(fname, parlist) {
   cl <- do.call("call", append(list(name = fname), parlist))
   return(format(cl))
@@ -285,7 +328,7 @@ prolongseq <- function(x, newrange, step=NULL) {
 intersect.ranges <- function(a, b, fatal=TRUE) {
   lo <- max(a[1],b[1])
   hi <- min(a[2],b[2])
-  if(lo >= hi) {
+  if(lo > hi) {
     if(fatal) stop("Intersection is empty")
     else return(NULL)
   }
@@ -293,20 +336,50 @@ intersect.ranges <- function(a, b, fatal=TRUE) {
 }
 
 inside.range <- function(x, r) {
-  stopifnot(length(r) == 2 && r[1] < r[2])
+  stopifnot(length(r) == 2 && r[1] <= r[2])
   return(x >= r[1] & x <= r[2])
 }
 
+check.in.range <- function(x, r, fatal=TRUE) {
+  xname <- deparse(substitute(x))
+  if(inside.range(x, r))
+    return(TRUE)
+  if(fatal) 
+    stop(paste(xname, "should be a number between",
+               r[1], "and", r[2]),
+         call.=FALSE)
+  return(FALSE)
+}
+
+startinrange <- function(x0, dx, r) {
+  ## find y = x0 + n * dx such that y \in r
+  if(all(inside.range(x0, r))) return(x0)
+  stopifnot(is.numeric(dx) && length(dx) == 1)
+  y <- x0 + dx * round((mean(r) - x0)/dx)
+  y[!inside.range(y, r)] <- NA
+  return(y)
+}
+
 prettyinside <- function(x, ...) {
   r <- range(x, na.rm=TRUE)
+  if(diff(r) == 0) return(r[1])
   p <- pretty(x, ...)
   ok <- inside.range(p, r)
   return(p[ok])
 }
 
+prettydiscrete <- function(x, n=10) {
+  nx <- length(x)
+  dx <- nx %/% n
+  if(dx < 1) return(x)
+  i <- 1 + (0:(n-1)) * dx
+  return(x[i])
+}
+
+
 check.range <- function(x, fatal=TRUE) {
   xname <- deparse(substitute(x))
-  if(identical(x, range(x, na.rm=TRUE)))
+  if(is.numeric(x) && identical(x, range(x, na.rm=TRUE)))
     return(TRUE)
   if(fatal) 
     stop(paste(xname, "should be a vector of length 2 giving (min, max)"))
@@ -320,9 +393,6 @@ niceround <- function(x, m=c(1,2,5,10)) {
   return(z)
 }
 
-assign(".Spatstat.ProgressBar", NULL, envir = .spEnv)
-assign(".Spatstat.ProgressData", NULL, envir = .spEnv)
-
 progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
                            nperline=min(charsperline,
                              every * ceiling(charsperline /(every+3))),
@@ -337,17 +407,16 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
          txtbar={
            if(i == 1) {
              # initialise text bar
-             assign(".Spatstat.ProgressBar",
-                    txtProgressBar(1, n, 1, style=3),
-                    envir = .spEnv)
+             putSpatstatVariable("Spatstat.ProgressBar",
+                                 txtProgressBar(1, n, 1, style=3))
            } else {
              # get text bar
-             pbar <- get(".Spatstat.ProgressBar", envir = .spEnv)
+             pbar <- getSpatstatVariable("Spatstat.ProgressBar")
              # update 
              setTxtProgressBar(pbar, i)
              if(i == n) {
                close(pbar)
-               assign(".Spatstat.ProgressBar", NULL, envir = .spEnv)
+               putSpatstatVariable("Spatstat.ProgressBar", NULL)
              } 
            }
          },
@@ -362,13 +431,12 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
              }
              showtime <- FALSE
              showevery <- n
-             assign(".Spatstat.ProgressData",
-                    list(every=every, nperline=nperline,
-                         starttime=now,
-                         showtime=FALSE, showevery=n),
-                    envir=.spEnv)
+             putSpatstatVariable("Spatstat.ProgressData",
+                                 list(every=every, nperline=nperline,
+                                      starttime=now,
+                                      showtime=FALSE, showevery=n))
            } else {
-             pd <- get(".Spatstat.ProgressData", envir=.spEnv)
+             pd <- getSpatstatVariable("Spatstat.ProgressData")
              if(is.null(pd))
                stop(paste("progressreport called with i =", i, "before i = 1"))
              every     <- pd$every
@@ -397,11 +465,10 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
                      showevery <- min(niceround(aminute), showevery)
                  }
                }
-               assign(".Spatstat.ProgressData",
-                    list(every=every, nperline=nperline,
-                         starttime=starttime,
-                         showtime=showtime, showevery=showevery),
-                    envir=.spEnv)
+               putSpatstatVariable("Spatstat.ProgressData",
+                                   list(every=every, nperline=nperline,
+                                        starttime=starttime,
+                                        showtime=showtime, showevery=showevery))
              }
            }
            if(i == n) 
@@ -787,6 +854,7 @@ sensiblevarname <- function(guess, fallback, maxlen=12) {
   return(out)
 }
 
+## deparse() can sometimes be equivalent to dumping the whole object
 short.deparse <- function(x, maxlen=60) {
   deparse(x,
           nlines=1,
@@ -794,6 +862,9 @@ short.deparse <- function(x, maxlen=60) {
           control="delayPromises")
 }
 
+## deparse() can produce multiple lines of text
+flat.deparse <- function(x) { paste(deparse(x), collapse=" ") }
+
 good.names <- function(nama, defaults, suffices) {
   # ensure sensible, unique names 
   stopifnot(is.character(defaults))
@@ -1006,7 +1077,41 @@ dotexpr.to.call <- function(expr, dot="funX", evaluator="eval.fv") {
   return(cc)
 }
 
-# print names and version numbers of libraries loaded
+inject.expr <- function(base, expr) {
+  ## insert an expression inside a call and parse it
+  txt <- sub(".", as.character(expr), as.character(base), fixed=TRUE)
+  parse(text=txt)
+}
+
+  
+## Match variable names to objects in 'data' list or environment
+getdataobjects <- function(nama, envir, datalist=NULL, fatal=FALSE) {
+  if(is.null(nama)) return(NULL)
+  stopifnot(is.character(nama))
+  n <- length(nama)
+  y <- vector(mode="list", length=n)
+  names(y) <- nama
+  if(!is.null(datalist)) {
+    hit <- nama %in% names(datalist)
+    if(any(hit))
+      y[hit] <- as.list(datalist)[nama[hit]]
+    external <- unlist(lapply(y, is.null))
+  } else external <- rep(TRUE, n)
+  y[external] <- mget(nama[external], envir=envir,
+                    ifnotfound=list(NULL), inherits=TRUE)
+  if(fatal && any(bad <- unlist(lapply(y, is.null)))) {
+    nbad <- sum(bad)
+    stop(paste(ngettext(nbad, "Covariate", "Covariates"),
+               commasep(sQuote(nama[bad])),
+               ngettext(nbad, "was not found", "were not found")),
+         call.=FALSE)
+  }
+  names(y) <- nama
+  attr(y, "external") <- external
+  return(y)
+}
+ 
+## print names and version numbers of libraries loaded
 
 sessionLibs <- function() {
   a <- sessionInfo()
@@ -1125,3 +1230,81 @@ ifelseNegPos <- function(test, x) {
   # b is evaluated only now
   return(b)
 }
+
+blockdiagmatrix <- function(...) {
+  x <- list(...)
+  if(!all(unlist(lapply(x, is.matrix))))
+    stop("Some of the arguments are not matrices", call.=FALSE)
+  nr <- unlist(lapply(x, nrow))
+  nc <- unlist(lapply(x, ncol))
+  result <- matrix(0, sum(nr), sum(nc))
+  rownames(result) <- unlist(lapply(x, rownames))
+  colnames(result) <- unlist(lapply(x, colnames))
+  rowend <- cumsum(nr)
+  rowstart <- c(0, rowend) + 1
+  colend <- cumsum(nc)
+  colstart <- c(0, colend) + 1
+  for(i in seq_along(x))
+    result[ (rowstart[i]):(rowend[i]) , (colstart[i]):(colend[i])] <- x[[i]]
+  return(result)
+}
+
+blockdiagarray <- function(...) {
+  x <- list(...)
+  if(!all(unlist(lapply(x, is.array))))
+    stop("Some of the arguments are not arrays", call.=FALSE)
+  dims <- lapply(x, dim)
+  dims1 <- unlist(lapply(dims, "[", i=1))
+  if(length(dim1 <- unique(dims1)) > 1)
+    stop("Arrays have different extents in first dimension")
+  dims2 <- unlist(lapply(dims, "[", i=2))
+  dims3 <- unlist(lapply(dims, "[", i=3))
+  result <- array(0, dim=c(dim1, sum(dims2), sum(dims3)))
+  dn <- lapply(x, dimnames)
+  dimnames(result)[[2]] <- unlist(lapply(dn, "[[", 2))
+  dimnames(result)[[3]] <- unlist(lapply(dn, "[[", 3))
+  rowend <- cumsum(dims2)
+  rowstart <- c(0, rowend) + 1
+  colend <- cumsum(dims3)
+  colstart <- c(0, colend) + 1
+  for(i in seq_along(x))
+    result[ , (rowstart[i]):(rowend[i]) , (colstart[i]):(colend[i])] <- x[[i]]
+  return(result)
+}
+
+lty2char <- function(i) {
+  if(is.numeric(i)) c("blank", "solid", "dashed", "dotted",
+                      "dotdash", "longdash", "twodash")[(i %% 7) + 1] else i
+}
+
+## convert numeric matrix to character, and blank out lower sub-diagonal.
+uptrimat <- function(x) {
+  stopifnot(is.matrix(x))
+  x[] <- as.character(x)
+  x[row(x) > col(x)] <- ""
+  return(noquote(x))
+}
+
+asNumericMatrix <- function(x) {
+  ## workaround for strange artefact of as.matrix.data.frame
+  x <- as.matrix(x)
+  storage.mode(x) <- "double"
+  x
+}
+
+prepareTitle <- function(main) {
+  ## Count the number of lines in a main title
+  ## Convert title to a form usable by plot.owin
+  if(is.expression(main)) {
+    nlines <- 1
+  } else {
+    main <- paste(main)
+    ## break at newline 
+    main <- unlist(strsplit(main, "\n"))
+    nlines <- if(sum(nchar(main)) == 0) 0 else length(main)
+  }
+  return(list(main=main,
+              nlines=nlines,
+              blank=rep('  ', nlines)))
+}
+
